@@ -55,7 +55,7 @@
 uint16_t MemRecursorCache::s_maxServedStaleExtensions;
 
 MemRecursorCache::MemRecursorCache(size_t mapsCount) :
-  d_maps(mapsCount)
+  d_maps(mapsCount == 0 ? 1 : mapsCount)
 {
 }
 
@@ -761,7 +761,7 @@ bool MemRecursorCache::updateValidationStatus(time_t now, const DNSName& qname,
   return updated;
 }
 
-uint64_t MemRecursorCache::doDump(int fd)
+uint64_t MemRecursorCache::doDump(int fd, size_t maxCacheEntries)
 {
   int newfd = dup(fd);
   if (newfd == -1) {
@@ -775,11 +775,17 @@ uint64_t MemRecursorCache::doDump(int fd)
 
   fprintf(fp.get(), "; main record cache dump follows\n;\n");
   uint64_t count = 0;
-
+  size_t shard = 0;
+  size_t min = std::numeric_limits<size_t>::max();
+  size_t max = 0;
   for (auto& mc : d_maps) {
     auto map = mc.lock();
+    const auto shardSize = map->d_map.size();
+    fprintf(fp.get(), "; record cache shard %zu; size %zu\n", shard, shardSize);
+    min = std::min(min, shardSize);
+    max = std::max(max, shardSize);
+    shard++;
     const auto& sidx = map->d_map.get<SequencedTag>();
-
     time_t now = time(nullptr);
     for (const auto& i : sidx) {
       for (const auto& j : i.d_records) {
@@ -802,6 +808,7 @@ uint64_t MemRecursorCache::doDump(int fd)
       }
     }
   }
+  fprintf(fp.get(), "; main record cache size: %zu/%zu shards: %zu min/max shard size: %zu/%zu\n", size(), maxCacheEntries, d_maps.size(), min, max);
   return count;
 }
 
